#################################################################################
# Created       : Martin Foster
# Modified      : 14-Dec-2006
#################################################################################
#
# Who - Modules part of Ethereal Realms designed to determine who is on 
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

package Ethereal::Who;						# Package name

use strict;							# Anal rententive variable check

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle
	my $tmpl     = shift;					# Template management
	my $param    = shift;					# Parameter list

	my $id;							# Post ID Number

	#####################
	# Program Area

	# Retrieve initial start point
	($id) = $database->DataGetPostID();

	# Setup hash
	my $self     = {
		HANDLE	=> $database,
		CGI	=> $cgi,
		TMPL    => $tmpl,
		PARAM   => $param,
		POST    => $id
	 };


	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Portal
#
# Simply lists the amount of people in a realm to gage activity without
# actually using the normal global who functionality.  Used at the portal
# specifically.

sub Portal
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $realm    = shift;					# Default realm name
	my $array    = shift;					# Array of entries
	my $hash     = shift;					# Hash of entries

	my $database = $self->{'HANDLE'};			# Database handle
	my $statement;						# Database statement handle

	my $def;						# Default realm
	my $count;						# Persons in realm
	my $name;						# Realm name
	my $pos;						# Position
	my $size;						# Size of array


	#####################
	# Data members

	# Get results
	$database->Pull(\$statement, "SELECT
		  RealmName AS \"Realm\",
		  COUNT(*)  AS \"Count\"
		FROM Portal
		GROUP BY RealmName
		ORDER BY RealmName"
	);

	# Get size
	$size = @{$array};

	# Loop through updated fields
	while (($name, $count) = $statement->fetchrow())
	{
		# Default realm
		if (($name eq $realm)
		 && ($count > 0))
		{
			# Allow script to function
			$def = "$name ($count)";
		}

		# Generate additional hash
		if (defined($hash))
		{
			# Throw in additional information
			$hash->{$name} = $count; 
		}
				

		# Search and replace
		for ($pos=0; $pos < $size; $pos++)
		{
			# Match found
			if ($array->[$pos] eq $name)
			{
				# Change value
				$array->[$pos] = "$name ($count)";

				# Skip
				last;
			}
		}
	}

	# Finish query
	$statement->finish();


	# Ensure definition
	$def = (defined($def)) ? $def : $array->[$pos];
	
	# Return default realm
	return $def;
}


#####################
# Pesky
#
# This is a variant of the regular private message retrieval system with the
# exception that it allows people to see more then they should.

sub Pesky
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $array    = shift;					# Number of entries
	my $hash     = shift;					# Key pair value
	my $realm    = shift;					# Search realm

	my $database = $self->{'HANDLE'};			# Database handle
	my $post     = $self->{'POST'};				# Post ID Number

	my $key;						# Key for generation
	my $res;						# Database results
	my $statement;						# Database statement handle


	#####################
	# Data members

	# Get results
	$database->Pull(\$statement, "SELECT
		  Pu.PuppetName AS \"PuppetName\"
		FROM Post Po, Puppet Pu, Realm Re
		WHERE Po.PuppetName     = Pu.PuppetName
		  AND Po.puppeteerlogin = Pu.PuppeteerLogin
		  AND Po.RealmName      = Re.RealmName
		  AND Pu.PuppetPool     = 'system'
		  AND Re.RealmPublic    = 'yes'
		  AND Po.PostIDNumber > ?::INT
		GROUP BY Pu.PuppetName
		ORDER BY Pu.PuppetName", $post);

	# Loop and build
	while ($res = $statement->fetchrow_hashref())
	{
		# Final key format
		$key = 'MAIN' . $res->{'PuppetName'};


		# Build up
		# Array
		push(@{$array}, $key);

		# Hash
		$hash->{$key} = $res->{'PuppetName'};
	}

	# Finish up
	$statement->finish();
}


#####################
# Private
#
# This will allow for the list of targets to be specified at realm level

sub Private
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $array    = shift;					# Number of entries
	my $hash     = shift;					# Key pair value
	my $realm    = shift;					# Search realm

	my $database = $self->{'HANDLE'};			# Database handle
	my $post     = $self->{'POST'};				# Post ID Number

	my $deco;						# Decorative pointer
	my $key;						# Pool/Puppet key
	my $res;						# Database results
	my $statement;						# Database statement handle


	#####################
	# Data members

	# Get results
	$database->Pull(\$statement, "SELECT
		  Pu.PuppetName AS \"PuppetName\",
		  Pu.PuppetPool AS \"PuppetPool\"
		FROM Post Po, Puppet Pu
		WHERE Po.PuppetName     = Pu.PuppetName
		  AND Po.puppeteerlogin = Pu.PuppeteerLogin
		  AND Po.RealmName=?
		  AND Po.PostIDNumber > ?::INT
		GROUP BY Pu.PuppetName, Pu.PuppetPool
		ORDER BY Pu.PuppetName",

	 $realm,
	 $post
	);

	# Loop and build
	while ($res = $statement->fetchrow_hashref())
	{
		# Format key
		# System handles
		if ($res->{'PuppetPool'} eq 'system')
		{
			# Key
			$key  = 'MAIN';

			# Decorative
			$deco = '';
		}

		# Canon realms
		else
		{
			# Key
			$key  = 'ROOM';

			# Decorative
			$deco = '*';
		}

		# Final key format
		$key = $key . $res->{'PuppetName'};


		# Build up
		# Array
		push(@{$array}, $key);

		# Hash
		$hash->{$key} = $res->{'PuppetName'} . $deco;
	}

	# Finish up
	$statement->finish();
}


#####################
# Who
#
# Simply lists who is on at the time

sub Who
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $choice   = shift;					# Choice of the matter

	my $database = $self->{'HANDLE'};			# Database handle
	my $cgi      = $self->{'CGI'};				# CGI handle
	my $param    = $self->{'PARAM'};			# Parameter list
	my $tmpl     = $self->{'TMPL'};				# Template management

	my $statement;						# Database statement handle

	my $css;						# Style sheet
	my $count;						# Colour change
	my $realm;						# Realms title
	my $super;						# Supervisory access
	my $who;						# Who is on

	my $entrance = '';					# Entrance widget
	my $listing  = '';					# Listing of objects
	my $whois    = '';					# Listing of who is on

	my @embed;						# Embeded parameters
	my @who;						# Who is on list

	my %realm;						# Realm hash


	#####################
	# Program area

	# Populate hash
	$database->GetHashRealm(\%realm);


	# Supervisory access
	($super) = $database->DataGet("SELECT p.PuppeteerLogin 
		FROM  Puppeteer p, RealmSupervisor r
		WHERE p.PuppeteerLogin=r.PuppeteerLogin
		  AND p.PuppeteerLogin=? 
                  AND r.RealmName=?", 

	 $param->{'USER'}, 
         $param->{'ROOM'}
	);


	# Prepare and execute
	# Regular users
	unless (defined($super))
	{
		# Normal Who
		$database->Pull(\$statement, "SELECT PuppetName AS \"PuppetName\"
			FROM Post
			WHERE RealmName=?
			  AND PostIDNumber > ?::INT
			  AND PuppetName IS NOT NULL
			GROUP BY PuppetName
			ORDER BY PuppetName",

		 $param->{'ROOM'},
		 $self->{'POST'}
		);
	}

	# Supervisory
	else
	{
		# System Who
                $database->Pull(\$statement, "SELECT
			PuppetName || ' (' || PuppeteerLogin || ') ' || getWarning(RealmName, PuppeteerLogin) AS \"PuppetName\"
			FROM Post
			WHERE RealmName=?
			  AND PostIDNumber > ?::INT
			  AND PuppetName IS NOT NULL
			GROUP BY PuppetName, PuppeteerLogin, RealmName
			ORDER BY PuppetName",

		 $param->{'ROOM'},
		 $self->{'POST'}
		);
	}


	# Set
	$count = 0;

	# Format line
	while ($who = $statement->fetchrow())
	{
		# Increment
		$count++;

		# Determine colour
		$css = (($count % 2) == 0)
		  ? 'WHOPRI'
		  : 'WHOSEC';

		# Append to listing
		$listing .= $tmpl->Pass('TmplWhoRoomList',
			MROW  => $css,
			MNAME => $cgi->escapeHTML($who)
		 );
	}

	# End statement
	$statement->finish();


	# Choice selection
	if ($choice < 99)
	{
		# Determin form
		push(@embed, $param->Flat()) if ($choice == 0);
		push(@embed, $param->Flat(), READONLY=>'True') if ($choice == 1);

		# Display choice
		# Form start
		$entrance .= $cgi->start_form(
			-method  => 'POST',
			-action  => $database->{'SYS'}{'LnkIntRealm'},
			-enctype => 'application/x-www-form-urlencoded'
		 );
		$entrance .= $param->EmbedNormal(@embed);

		# Append widget
		$entrance .= $cgi->submit(
			-name   => $realm{'TxtRealmEnter'},
			-value  => $realm{'TxtRealmEnter'},
			-class  => 'ENTRANCE'
		 );

		# Form end
		$entrance .= $cgi->end_form() . "\n";
	}

	# Generate template
	$whois = $tmpl->Pass('TmplWhoRoom',
		LSTWHO => $listing
	 );
		
	# Return everything
	return $whois, $entrance;
}

# Alias for Entrace
sub WhoEntrance
{ return Who(shift, 0); }

# Alias for Normal
sub WhoGuest
{ return Who(shift, 1);}

# Alias for Normal
sub WhoNormal
{ return Who(shift, 99);}



#####################
# System/Golbal Who
#
# Simply lists who is on at the time

sub WhoAdvanced	
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $choice   = shift;					# Choice of the matter

	my $database = $self->{'HANDLE'};			# Database handle
	my $cgi      = $self->{'CGI'};				# CGI handle
	my $tmpl     = $self->{'TMPL'};				# Templates

	my $statement;						# Database statement handle

	my $listing  = '';					# Listing of objects
	my $whois    = '';					# Listing of who is on

	my $css;						# Colour change
	my $count    = 0;					# Counter
	my $who;						# Who is on


	#####################
	# Program area

	# Prepare and execute
	unless (defined($choice))
	{
		# Normal Who
		$database->Pull(\$statement, "SELECT
			  Po.PuppetName AS \"PuppetName\",
			  Po.RealmName  AS \"RealmName\" 
			FROM Post Po, Realm Re
			WHERE Po.RealmName = Re.RealmName
			  AND Re.RealmPublic='yes'
			  AND Po.PostIDNumber > ?::INT
			  AND Po.PuppetName IS NOT NULL
			GROUP BY Po.RealmName, Po.PuppetName
			ORDER BY Po.RealmName, Po.PuppetName",

		 $self->{'POST'}
		);
	}
	else
	{
		# Normal Who
		$database->Pull(\$statement, "SELECT
			  PuppetName || ' (' || PuppeteerLogin || ') ' || getWarning(RealmName, PuppeteerLogin) AS \"PuppetName\",
			 RealmName  AS \"RealmName\" 
			FROM Post
			WHERE PostIDNumber > ?::INT
			  AND PuppetName IS NOT NULL
			GROUP BY RealmName, PuppetName, PuppeteerLogin
			ORDER BY RealmName, PuppetName",

		 $self->{'POST'}
		);
	}

	# Format line
	while ($who = $statement->fetchrow_hashref())
	{
		# Increment counter
		$count++;

		# Determine colour
		$css = (($count % 2) == 0)
		  ? 'WHOPRI'
		  : 'WHOSEC';

		# Append to listing
		$listing .= $tmpl->Pass('TmplWhoSiteList',
			MROW  => $css,
			MNAME => $cgi->escapeHTML($who->{'PuppetName'}),
			MROOM => $cgi->escapeHTML($who->{'RealmName'}) 
		 );
	}

	# Finish Query
	$statement->finish();

	# Generate template
	return $tmpl->Pass('TmplWhoSite',
		LSTWHO  => $listing,
		MTOTALS => $count
	 );
}

# Alias for Who System
sub WhoSystem
{ return WhoAdvanced(shift, 1); };

1;								# Return true
